
Ti.API.info('== Loading Procs');
var Procs=exports;

// Crop year definitions (month that the new crop year starts in)
Procs.cccCROPYEAR  = 6;
Procs.usdaCROPYEAR = 7;
Procs.cmaCROPYEAR  = 8;


exports.DebugMsg=function(TheMsg)
{
  // Displays TheMsg in the Titanium Studio console window (Android Emulator
  // or iPhone simulator console) with asterisks surrounding it so that it can be
  // easily located amoungst the sea of emulator/simulator generated messages.
  // When DevelopmentMode is false (for releases), all calls are suppressed.
  //
  // Parameters:
  //   TheMsg:string
  //     The message to display in the console window.

  if (ApplicationGet('DevelopmentMode'))
  {
    Ti.API.info('********************************************');
    Ti.API.info('*--> '+TheMsg);
    Ti.API.info('********************************************');
  }
};

exports.DP=function(ThePixels)
{
  // Returns a string with the number ThePixlels plus 'dp'.
  //
  // Parameters:
  //   ThePixels:integer
  //     Number to be converted to a string.
  //     Example: 30
  // Returns:string
  //   Example: '30dp'
  // Usage:
  //   MyButton.width=DP(30);

  if (ThePixels != null)
    return String(ThePixels)+'dp';
  else
    return ThePixels; // null
};


exports.IfIsNull=function(TheObj,TheDefault)
{
  // If TheObj is null, returns TheDefault.  Otherwise, returns TheObj itself.
  // TheObj can be any type.  Commonly used to provide a default value of an
  // optional parameter.
  //
  // Parameters:
  //   TheObj:any type
  //     Value to be returned if not null.
  //     Example: 2
  //     Example: null
  //   TheDefault:any type
  //     Value to be returned if TheObj is null.
  //     Example: 30
  //     Example: 30
  // Returns:type of TheObj or TheDefault
  //   Example: 2
  //   Example: 30
  // Usage:
  //   MyButton.top=IfIsNull(TheTop,30);

  if (TheObj != null)
    return TheObj;
  else
    return TheDefault;
};


exports.ByRef=function(_X)
{
  // JavaScript does not allow parameters to be passed by reference...this function
  // works around that limitation.  ByRef takes a value and places a copy of it
  // into an object.  You then pass the resulting object to the function that wants
  // a by-reference parameter.  See NextDelimitedWord() for a prime example.
  //
  // Parameters:
  //   _X:any type
  //     The value/variable that you want to pass by reference.
  // Returns:object
  // Usage:
  //   var O1=Procs.NextDelimitedWord(I1=Procs.ByRef("one,TWO,three,FOUR"),',');
  //   var O2=Procs.NextDelimitedWord(I1,',');
  //   Messages.AtMsg('The first two words of I1 are '+O1+' and '+O2);
  //   Messages.AtMsg('The remaining words of I1 are '+I1.Value);

  return {Value:_X};
};


exports.Trim=function(TheStr)
{
  // Trims a string of any leading or trailing spaces.
  //
  // Parameters:
  //   TheStr:string
  //     The string to trim.
  // Returns:string

  return String(TheStr).replace(/^\s+|\s+$/g,"");
};


exports.LTrim=function(TheStr)
{
  // Trims a string of any leading spaces (left trim).
  //
  // Parameters:
  //   TheStr:string
  //     The string to trim.
  // Returns:string

  return String(TheStr).replace(/^\s+/,"");
};


exports.RTrim=function(TheStr)
{
  // Trims a string of any trailing spaces (right trim).
  //
  // Parameters:
  //   TheStr:string
  //     The string to trim.
  // Returns:string

  return String(TheStr).replace(/\s+$/,"");
};


exports.SEmpty=function(TheStr)
{
  // Returns whether a string is "empty", which means it is either null, undefined,
  // a 0-length string, or a string that contains only blank spaces.
  //
  // Parameters:
  //   TheStr:string
  //     The string to check.
  // Returns:boolean

  return (TheStr == null) || (TheStr == undefined) || (TheStr.length == 0) || (Procs.Trim(TheStr).length == 0);
};


exports.SLeft=function(TheStr,NumChars)
{
  // Returns the leftmost NumChars of TheStr.  If NumChars is negative, it returns
  // the leftmost part of the string, except the rightmost NumChars characters.
  //
  // Parameters:
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  //     Example: 'Hello World'
  //   NumChars:integer
  //     Positive value: The number of characters to return from the beginning of
  //     TheStr.
  //     Negative value: The number of rightmost characters of the string to not
  //     return.
  //     Example: 5
  //     Example: -4
  // Returns:string
  //   Example: 'Hello'
  //   Example: 'Hello W'

  if (NumChars >= 0)
    return TheStr.substr(0,NumChars);
  else
    return TheStr.substr(0,TheStr.length+NumChars);
};


exports.SRight=function(TheStr,NumChars)
{
  // Returns the rightmost NumChars of TheStr.
  //
  // Parameters:
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  //   NumChars:integer
  //     The number of characters to return from the right end of TheStr.
  //     Example: 5
  // Returns:string
  //   Example: 'World'

  return TheStr.substr(TheStr.length-NumChars);
};


exports.Copy=function(TheStr,StartPos,EndPos)
{
  // Returns a substring of TheStr, starting at StartPos (1-based) and ending at
  // EndPos (1-based).  Note that EndPos is not the number of characters...it is
  // the ending position within TheStr.
  //
  // Parameters:
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  //   StartPos:integer
  //     The starting character number from TheStr to return.  The first character
  //     of TheStr is in position 1.
  //     Example: 5
  //   EndPos:integer
  //     The ending character number from TheStr to return.  The first character
  //     of TheStr is in position 1, and the last character of TheStr is in
  //     position TheStr.length.
  //     Example: 8
  // Returns:string
  //   Example: 'o Wo'

  return TheStr.substr(StartPos-1,EndPos-StartPos+1);
};


exports.Pos=function(TheFind,TheStr)
{
  // Returns the position (1-based) of TheFind in TheStr.
  //
  // Parameters:
  //   TheFind:string
  //     The string to find within TheStr.
  //     Example: 'ell'
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  // Returns:integer
  //   Example: 2

  return (TheStr.indexOf(TheFind) + 1);
};


exports.RestStr=function(TheStr,StartPos)
{
  // Returns the rest of the string TheStr, starting at StartPos (1-based).
  //
  // Parameters:
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  //   StartPos:integer
  //     The starting character number from TheStr to return.  The first character
  //     of TheStr is in position 1.
  //     Example: 2
  // Returns:string
  //   Example: 'ello World'

  return TheStr.substr(StartPos-1);
};


exports.StrUpTo=function(TheStr, TheChr, OrTheWholeThing)
{
  // Returns the leftmost portion of TheStr, up until TheChr is encountered.  If
  // TheChr does not exist in TheStr, an empty string will be returned, unless
  // OrTheWholeThing is true, in which case the entire TheStr is returned.
  //
  // Parameters:
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  //     Example: 'Hello World'
  //   TheChr:char,string
  //     The character or string that stops the return value accumulation.
  //     Example: 'r'
  //     Example: 'Monkey'
  //   OrTheWholeThing:boolean (optional...defaults to false)
  //     Pass true if you want the entire TheStr returned if TheChr doesn't exist
  //     in TheStr at all.
  //     Example: (null)
  //     Example: true
  // Returns:string
  //   Example: 'Hello Wo'
  //   Example: 'Hello World'

  var UpToChars = Math.max(Procs.Pos(TheChr,TheStr)-1,0);
  if ((UpToChars == 0) && OrTheWholeThing)
    return TheStr;
  else
    return Procs.SLeft(TheStr,UpToChars);
};


exports.StrTran=function(TheStr, TheFind, TheReplace)
{
  // Returns TheStr with all occurrences of TheFind replaced with the string
  // TheReplace.  If TheReplace is not passed, all occurrences of TheFind are
  // removed.
  //
  // Parameters:
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  //     Example: 'Hello World'
  //   TheFind:char,string
  //     The character or string to find.  Be careful, this is treated as
  //     a regular expression, so all regular expression escaping rules apply!
  //     Example: 'l'
  //     Example: 'o'
  //   TheReplace:string (optional...defaults to '')
  //     The string that should take the place of all occurences of TheFind.
  //     Example: 'y'
  //     Example: (null)
  // Returns:string
  //   Example: 'Heyyo Woryd'
  //   Example: 'Hell Wrld'

  if (TheReplace == null)
    TheReplace='';
  var Pattern=new RegExp(TheFind,'g');
  return String(TheStr).replace(Pattern, TheReplace);
};


exports.RemoveChars=function(CharsToRemove, TheString)
{
  // Returns TheString with all occurrences of any of the characters in
  // CharsToRemove removed.
  //
  // Parameters:
  //   CharsToRemove:char,string
  //     The character(s) to remove.
  //     Example: 'lo'
  //   TheString:string
  //     The original string.
  //     Example: 'Hello World'
  // Returns:string
  //   Example: 'He Wrd'

  Result=TheString;
  for (var i=0; i < CharsToRemove.length-1; i++) {
    Result=Procs.StrTran(Result,CharsToRemove.charAt(i));
  };
  return Result;
};


exports.SpaceKill=function(TheStr)
{
  // Returns TheStr with all spaces removed.
  //
  // Parameters:
  //   TheStr:string
  //     The original string.
  //     Example: 'Hello World'
  // Returns:string
  //   Example: 'HelloWorld'

  return Procs.StrTran(TheStr,' ','');
};


exports.NextDelimitedWord=function(ByRef_Phrase, Delims, TrimDupDelims)
{
  // Returns the leftmost part of the Value of ByRef_Phrase (see ByRef()), stopping
  // at any of the characters in Delims.  If TrimDupDelims is true, then any
  // characters from Delims that remain at the leftmost of the string will also be
  // removed.  After the call to NextDelimitedWord, the Value of ByRef_Phrase will
  // be the remainder of the string after the return value (and any duplicate
  // delimiters) have been removed.
  //
  // Parameters:
  //   ByRef_Phrase:ByRef(string)
  //     The original phrase.
  //     Example: Input=Procs.ByRef('one,,TWO,three,FOUR')
  //   Delims:char,string
  //     The character(s) that should be used as delimiters
  //     Example: ','
  //   TrimDupDelims:boolean (optional...defaults to false)
  //     If true, then any characters from Delims that remain at the leftmost of
  //     the return string will also be removed.
  //     Example: true
  // Returns:string
  //   Example: 'one' (and Input.Value will contain 'TWO,three,FOUR')

  var FirstWord;
  var TheDelimiter='';
  var WordDelim=99999;
  var NumDelims=Delims.length;
  var DelimPos=new Array(NumDelims);
  for (var d=0; (d < NumDelims); d++)
  {
    DelimPos[d]=Procs.Pos(Delims.charAt(d),ByRef_Phrase.Value);
    if ((DelimPos[d] > 0) && (DelimPos[d] < WordDelim))
      WordDelim=DelimPos[d];
  }
  if (WordDelim < 99999)
  {
    TheDelimiter=ByRef_Phrase.Value.charAt(WordDelim-1); // charAt is 0 based
    FirstWord=Procs.SLeft(ByRef_Phrase.Value,WordDelim-1);
    ByRef_Phrase.Value=Procs.RestStr(ByRef_Phrase.Value,WordDelim+1);
    if (TrimDupDelims)
      while (Procs.Pos(ByRef_Phrase.Value.charAt(0),Delims) > 0)
        ByRef_Phrase.Value=Procs.RestStr(ByRef_Phrase.Value,2);
  }
  else
  {
    FirstWord=ByRef_Phrase.Value;
    ByRef_Phrase.Value='';
    TheDelimiter='';
  }
  return FirstWord;
};


exports.IsAllCaps=function(TheStr)
{
  // Returns whether TheStr contains only non-lowercase characters.  Characters
  // outside of the alphabet are ignored by this function.
  //
  // Parameters:
  //   TheStr:string
  //     The string to examine.
  //     Example: '1 For All'
  //     Example: 'ALL FOR 1'
  // Returns:boolean
  //   Example: false
  //   Example: true

  return (TheStr.toUpperCase() == TheStr);
};


exports.IVal=function(TheStr)
{
  // Returns the integer value of TheStr.  Examination stops at the first
  // non-numeric character.
  //
  // Parameters:
  //   TheStr:string
  //     The string to value.
  //     Example: '123'
  //     Example: '120dp'
  //     Example: 'huh?'
  // Returns:integer
  //   Example: 123
  //   Example: 120
  //   Example: 0

  var Result=parseInt(TheStr);
  if (isNaN(Result))
    Result=0;
  return Result;
};


exports.RVal=function(TheStr)
{
  // Returns the real (double-precision floating point) value of TheStr.
  // Examination stops at the first non-numeric character (except for the decimal
  // point).
  //
  // Parameters:
  //   TheStr:string
  //     The string to value.
  //     Example: '123.45'
  //     Example: '120.1dp'
  //     Example: 'huh?'
  // Returns:integer
  //   Example: 123.45
  //   Example: 120.1
  //   Example: 0

  var Result=parseFloat(TheStr);
  if (isNaN(Result))
    Result=0;
  return Result;
};


exports.Replicate=function(TheStr,Size)
{
  // Returns TheStr repeated Size times.  Uses a folding algorithm for extremely
  // fast speed.
  //
  // Parameters:
  //   TheStr:string
  //     The string to replicate.
  //     Example: 'No gum in school'
  //   Size:integer
  //     The number of times to repeat TheStr.
  //     Example: 3
  // Returns:string
  //   Example: 'No gum in schoolNo gum in schoolNo gum in school'

  var Result='';
  while (Size > 0)
  {
    if (Size & 1)
      Result+=TheStr;
    Size>>=1;
    TheStr+=TheStr;
  }
  return Result;
};


exports.ZeroFill=function(Num,NumLen)
{
  // Returns a string containing Num with enough leading zeros to make the
  // resulting string NumLen characters long.
  //
  // Parameters:
  //   Num:integer,string
  //     The number or string to return right-justified in a zero filled string of
  //     NumLen characters.
  //     Example: 'ID'
  //     Example: 256
  //   NumLen:integer
  //     The length of the resulting string.
  //     Example: 5
  //     Example: 5
  // Returns:string
  //   Example: '000ID'
  //   Example: '00256'

  var NumString = Num + '';
  while(NumString.length < NumLen)
    NumString = "0" + NumString;
  return NumString;
};


exports.ZeroFillBlanks=function(Num,NumLen)
{
  // Returns the ZeroFill() of Num in a string NumLen characters long, unless Num
  // is 0, in which case it returns NumLen spaces (zeros as blanks).
  //
  // Parameters:
  //   Num:integer,string
  //     The number or string to return right-justified in a zero filled string of
  //     NumLen characters.
  //     Example: 'ID'
  //     Example: 256
  //     Example: 0
  //   NumLen:integer
  //     The length of the resulting string.
  //     Example: 5
  //     Example: 5
  //     Example: 5
  // Returns:string
  //   Example: '000ID'
  //   Example: '00256'
  //   Example: '     '

  if (Num == 0)
    return Procs.Replicate(' ',NumLen);
  else
    return Procs.ZeroFill(Num,NumLen);
};

exports.PhoneNumberFormat = function(PhoneNumber)
{
  // Returns a phone number formated as "(123)456-7890"
  // Parameters:
  //   PhoneNumber:integer,string
  //     Example: 3156353426
  //     Example: '3156353426'
  //     Example: '315-635-3426'
  //     Example: '(315)635-3426'
  // Returns:string
  //     Example: '(315)635-3426'

  PhoneNumber = Procs.Trim(PhoneNumber);

  // Remove any current formatting
  var InvalidCharacterSet = /[() \-+*#]/g;
  if (InvalidCharacterSet.test(PhoneNumber))
    PhoneNumber = PhoneNumber.replace(InvalidCharacterSet, '');

  if (PhoneNumber.length >= 10)
    PhoneNumber = '(' + PhoneNumber.substr(0, 3) + ')' +
                        PhoneNumber.substr(3, 3) + '-' +
                        PhoneNumber.substr(6, 4);
  return PhoneNumber;
}

exports.TypeOf=function(TheObject)
{
  // Returns the object type of TheObject, as a string.  Note that this is an
  // expensive (slow) call.
  //
  // Parameters:
  //   TheObject:object
  //     The object to examine.
  //     Example: MyListBox.data[0]
  // Returns:string
  //   Example: '[object Ti.UI.TableViewSection]'

  return Object.prototype.toString.call(TheObject);
};


exports.Plural=function(TheStr, Things)
{
  // Returns the simple plural term of TheStr, by adding an 's' to the end if
  // Things is not exactly 1.
  //
  // Parameters:
  //   TheStr:string
  //     The term (noun) that describes the Thing(s) in question.
  //     Example: 'ball'
  //     Example: 'ball'
  //   Things:integer
  //     The number of TheStr that you have.
  //     Example: 3
  //     Example: 1
  // Returns:string
  //   Example: 'balls'
  //   Example: 'ball'

  return ((Things == 1) ? TheStr : TheStr + 's');
};


exports.CropYear=function(TheDate,CropYearDefiner)
{
  // Returns the crop year that contains TheDate, using the definition of a crop
  // year as defined by CropYearDefiner.
  //
  // Parameters:
  //   TheDate:date
  //     The date in question.
  //     Example: 05/30/2011
  //   CropYearDefiner:crop year constant (optional...defaults to Procs.usdaCROPYEAR)
  //      Defines the month that represents the first month of the crop year
  //      (choose from Procs.cccCROPYEAR, Procs.usdaCROPYEAR, or Procs.cmaCROPYEAR).
  //     Example: Procs.cccCROPYEAR
  // Returns:integer
  //   Example: 2010

  if (CropYearDefiner == null)
    CropYearDefiner=Procs.usdaCROPYEAR;
  return (TheDate.getFullYear() - (((TheDate.getMonth()+1) < CropYearDefiner) ? 1 : 0));
};


exports.DTOC2=function(TheDate)
{
  // Converts a date to a string in the format MM/DD/YYYY.
  //
  // Parameters:
  //   TheDate:date
  //     The date to convert.
  //     Example: 5/30/2011
  // Returns:string
  //   Example: '05/30/2011'

  var Result;
  if (TheDate)
    Result=Procs.ZeroFill(TheDate.getMonth()+1,2)+'/'+ // getMonth returns 0-11
                          Procs.ZeroFill(TheDate.getDate(),2)+'/'+
                          TheDate.getFullYear();
  else
    Result='';
  return Result;
};


exports.DRound=function(TheRealNum,ToDigits)
{
  // Rounds a real number to ToDigits after the decimal point.
  //
  // Parameters:
  //   TheRealNum:real
  //     The real number to round.
  //     Example: 123.456789
  //   ToDigits:integer
  //     The number of digits the result should have after the decimal.
  //     Example: 2
  // Returns:real
  //   Example: 123.46

  var DoubleShift=Math.pow(10,ToDigits*2);
  var DoublePrecision=Math.round(TheRealNum * DoubleShift) / DoubleShift;
  var Shift=Math.pow(10,ToDigits);
  return Math.round(DoublePrecision * Shift) / Shift;
};


exports.IsNumeric=function(val)
{
  return !isNaN(parseFloat(val));
};


exports.NewStringBuilder=function()
{
  // Creates a StringBuilder used for concatenation of large
  // numbers of small strings.
  //
  // Returns:object
  //
  // Usage:
  //    var sb = Procs.createStringBuilder();
  //    while(<loop condition>) {
  //      sb.append("somestring");
  //    }
  //    sb.finalize();
  //    DebugMsg(sb.toString());
  //    sb.close();
  //
  //    sb.append(string) will append to the internal string
  //    sb.finalize() must be called before calling sb.toString()
  //    sb.toString() will return the string.  If used before finalize, garbage may be returned.
  //    sb.close()  releases resources used by the StringBuilder.
  //    sb.length is an integer representing the current length of the finalized string
  //

  var sb = {};
  sb.ibufpos = 0;
  sb.intstrbuff = [];
  sb.buffer = Titanium.createBuffer({ length: 16 });
  sb.timesexpanded = 4;
  sb.internal_resetlength = 0;
  sb.length = 0;
  sb.close = function()
  {
    this.buffer.release();
  };

  sb.append = function(appendstring)
  {
    this.intstrbuff.push(appendstring);
    this.length += appendstring.length;
    if(this.intstrbuff.length < 64)
      return;
    var tmpappend = this.intstrbuff.join("");
    this.intstrbuff = [];
    this.internal_append(tmpappend);
  };

  sb.internal_append = function(appendstring)
  {
    if(this.internal_resetlength > 0)
    {
      //Ti.API.info("Reset buffer length to " + this.internal_resetlength);
      this.buffer.length = this.internal_resetlength;
      this.internal_resetlength = 0;
    }
    while(this.buffer.length - this.ibufpos < appendstring.length) //extend buffer
    {
       if(this.timesexpanded < 17)
         this.timesexpanded++;  //max 131k chunk allocations
       //Ti.API.info("StringBuilder: Expanded buffer " + Math.pow(2, this.timesexpanded).toString() + " to size " + this.buffer.length.toString() + " to accomodate string of size " + appendstring.length.toString() + " at location " + this.ibufpos.toString());
       this.buffer.length = this.buffer.length  + Math.pow(2, this.timesexpanded);
    }
    Titanium.Codec.encodeString({ source: appendstring, dest:this.buffer, destPosition:this.ibufpos });
    this.ibufpos += appendstring.length;
  };

  sb.toString = function()
  {
    if(this.ibufpos == 0 && this.intstrbuff.length == 0)
      return "";
    if(this.ibufpos == 0 && this.intstrbuff.length > 0)
      return this.intstrbuff.join("");
    this.internal_resetlength = this.buffer.length; //Save expanded length of buffer
    this.buffer.length = this.ibufpos;
    return this.buffer.toString();
  };
  sb.finalize = function()
  {
    if(this.intstrbuff.length > 0)
    {
      if(this.ibufpos == 0) //exit early for only things in intstrbuff (buffer not even used)
        return;
      this.internal_append(this.intstrbuff.join(""));
    }
  };

  sb.truncate = function()
  {
    this.ibufpos = 0;
    this.intstrbuff = [];
    this.length = 0;
  };

  return sb;
};
